# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003-2004 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from Common import *

import os
import os.path
import ConfigParser
import string
import sys

CONF_TYPE_INTEGER         = 1 # An integer
CONF_TYPE_BOOLEAN         = 2 # A boolean (True, False, Yes, No, 1, 0)
CONF_TYPE_FLOAT           = 3 # A float
CONF_TYPE_STRING          = 4 # A string
CONF_TYPE_DIR             = 5 # One directory
CONF_TYPE_PATH            = 6 # Multiple directories
CONF_TYPE_FILE            = 7 # One file
CONF_TYPE_MULTIPLE_FILES  = 8 # Multiple files
CONF_TYPE_CHOICE          = 9 # Choose one value from a list.
CONF_TYPE_MULTIPLE_CHOICE = 10 # Choose several values froma a list.

# {"section": [(varname, description, type, defaultvalue, extra_info), ...], ...}
configuration_info = {
    "General": [("language", N_("The language to use Mattricks in"),
                 CONF_TYPE_CHOICE, "", [("", N_("Default language")),
                                        ("C", N_("English")),
                                        ("sv", N_("Swedish")),
                                        ("pl", N_("Polish")),
                                        ("nl", N_("Dutch")),
                                        ("de", N_("German"))]),
                ("hrf directories", N_("Catalogs with hrf files in"),
                 CONF_TYPE_PATH, [], None),
                ("lineup direction", N_("Direction for lineups"),
                 CONF_TYPE_CHOICE, "up", [("up", N_("Up")),
                                          ("down", N_("Down")),
                                          ("left", N_("Left")),
                                          ("right", N_("Right"))]),
                ("warning on exit", N_("Warning when exiting"),
                CONF_TYPE_BOOLEAN, False, None)],
    "Fetch": [("save directory", N_("Catalog to save hrf files in"),
               CONF_TYPE_DIR, os.path.realpath("."), None),
              ("download at startup", N_("Download at startup of Mattricks"),
               CONF_TYPE_BOOLEAN, False, None),
              ("always download", N_("Always download a new hrf file"),
               CONF_TYPE_BOOLEAN, True, None),
              ("override old hrf", N_("Overwrite day old hrf file if that exists"),
               CONF_TYPE_BOOLEAN, False, None),
              ("junior spy", N_("Use Junior Spy"),
               CONF_TYPE_BOOLEAN, False, None),
              ("ip number", N_("IP-number for the server"),
               CONF_TYPE_CHOICE, "195.149.159.156", [("195.149.159.155", "195.149.159.155"),
                                                     ("195.149.159.156", "195.149.159.156"),
                                                     ("195.149.159.157", "195.149.159.157"),
                                                     ("195.149.159.158", "195.149.159.158")])],
    "Team info": [("logo file", N_("Logotype file"),
                   CONF_TYPE_FILE, "", None)],
    "Player list": [("custom playerlist columns", N_("Custom columns for the playerlist"),
                     CONF_TYPE_MULTIPLE_CHOICE,
                     ["name", "ts_best_position", "warnings", "ska",
                     "rating", "for", "gttot"],
                     ["name", "playerID", "ald", "CountryID", "warnings", "ska",
                      "rating", "for", "TrainerSkill", "uth", "mlv", "spe",
                      "fra", "ytt", "bac", "mal", "fas", "rut", "led",
                      "sal", "mkt", "speciality", "gentleness", "honesty",
                      "Aggressiveness", "gtl", "gtc", "gttot", "gtt",
                      "gtall", "gev", "hat", "ts_best_position",
                      "ts_goalie", "ts_centraldefender",
                      "ts_centraldefender_offensive",
                      "ts_centraldefender_towards_the_wing", "ts_wingback",
                      "ts_wingback_offensive", "ts_wingback_defensive",
                      "ts_wingback_towards_the_middle", "ts_innermid",
                      "ts_innermid_offensive", "ts_innermid_defensive",
                      "ts_innermid_towards_the_wing", "ts_winger",
                      "ts_winger_offensive", "ts_winger_defensive",
                      "ts_winger_towards_the_middle", "ts_forward",
                      "ts_forward_defensive"])],
    "Match viewer": [("panel ratios", N_("Size ratios for events and report panels"),
                      CONF_TYPE_CHOICE, "1:2", [("1:2","1:2 (" + N_("larger report panel") + ")"),
                                                ("1:1","1:1 (" + N_("equal size") + ")"),
                                                ("2:1","2:1 (" + N_("larger event panel") + ")")]),
                     ("report direction", N_("Direction of events for match report"),
                      CONF_TYPE_CHOICE, "normal", [("normal",N_("Normal (latest event last)")),
                                                   ("reversed",N_("Reversed (latest event first)"))]),
                     ("remove warning", N_("Warning when removing game"),
                      CONF_TYPE_BOOLEAN, False, None),
                     ("detailed time", N_("Show detailed time until match start"),
                      CONF_TYPE_BOOLEAN, False, None)],

    }

def get_win_path(makedir = False):
    """Return the catalog for config files on Windows systems.

     - makedir -- if the program should create the catalog if it doesn't
                  exist.
    """

    _program = "Mattricks"
    try:
        from win32com.shell import shell, shellcon
        _path = shell.SHGetFolderPath(0, shellcon.CSIDL_APPDATA, None, 0)
        if isinstance(_path, unicode):
            encoding = sys.getfilesystemencoding()
            _path = _path.encode(encoding)
    except ImportError, e:
        _path = os.getenv("HOME")

    _path = os.path.join(_path, _program)
    if not os.path.exists(_path) and makedir:
        os.makedirs(_path)

    return _path

def get_load_filename(file, old = False):
    """Return the path to a config file for loading.

     - file -- the name of the config file.
     - old -- use the old (using ${HOME} instead of choices) system.

    If we are on windows, use the result from get_win_path, otherwise use
    the choices system. If old is True, then use ${HOME} instead of the
    choices system. On windows, old does nothing.
    """

    if sys.platform.startswith("win"):
        _path = get_win_path()
        return os.path.join(_path, file)

    # For non-windows systems

    if not old:
        import choices

        return choices.find_first("Mattricks", file)
    else:
        _path = os.getenv("HOME")
        return os.path.join(_path, file)

def get_database_filename(file):
    """Return the path to a config file for loading.

     - file -- the name of the config file.

    Returns None, if we can't find a suitable file.

    If we are on windows, use the result from get_win_path, otherwise use
    the choices system.
    """

    if sys.platform.startswith("win"):
        _path = get_win_path(True)
        return os.path.join(_path, file)

    # For non-windows systems

    import choices

    fn = choices.find_save("Mattricks", file)

    if fn is None:
        return None

    dir = os.path.dirname(fn)
    if not os.path.exists(dir):
        os.makedirs(dir)

    return fn

def save_cfg_file(file, cfg):
    """Write content to a config file.

     - file -- the name of the config file.
     - cfg -- the new contents of the config file.

    If we are on windows, use the result from get_win_path, otherwise use
    the choices system."""

    if sys.platform.startswith("win"):
        _path = get_win_path(True)
        fn = os.path.join(_path, file)
        fn_tmp = file + '.tmp'
        fp = open(fn_tmp, 'w')
    else:
        import choices

        fn, fn_tmp, fp = choices.open_save("Mattricks", file)

    if fp is not None:
        fp.write(cfg)
        fp.close()
        try:
            os.remove(fn)
        except:
            pass
        os.rename(fn_tmp, fn)

class Config:
    """Class for handling configuration of a program."""

    def __init__(self, config_info):
        """Initialization of the config.

         - config_info -- structure describing valid configuration options."""
        self.config_info = config_info

        filename = get_load_filename(self.__get_filename())

        if filename is not None:
            self.configuration = self.__read_config_file(filename)
        else:
            old_conf_file = get_load_filename(self.__get_filename(True), True)
            if os.path.exists(old_conf_file):
                self.configuration = self.__read_config_file(old_conf_file)
            else:
                self.configuration = self.__defaults()

    def save(self):
        ci = self.config_info.items()
        ci.sort()

        cfg = ""

        first = True
        for section, vars in ci:
            if first:
                first = False
            else:
                cfg += "\n"
            cfg += "[%s]\n" % section
            for varname, descr, type, default, extra_info in vars:
                cfg += "%s = %s\n" % \
                       (varname, self.__convert_value_to_string \
                        (type, self.configuration[section][varname]))

        filename = self.__get_filename()
        save_cfg_file(filename, cfg)

    def get(self, section, option):
        return self.configuration[section][option]

    def set(self, section, option, value):
        self.configuration[section][option] = value

    def __get_filename(self, old = False):
        if sys.platform.startswith("win"):
            fname = "mattricks.rc"
        else:
            if old:
                fname = ".mattricksrc"
            else:
                fname = "mattricksrc"

        return fname

    def __read_config_file(self, file):
        c = ConfigParser.ConfigParser()
        c.read(file)

        result = {}

        for section, vars in self.config_info.items():
            varlist = {}
            if not c.has_section(section):
                for varname, descr, type, default, extra_info in vars:
                    varlist[varname] = default
            else:
                for varname, descr, type, default, extra_info in vars:
                    if c.has_option(section, varname):
                        try:
                            varlist[varname] = self.__convert_string_to_value \
                                               (type, c.get(section, varname), extra_info)
                        except ValueError, error:
                            sys.stderr.write("Error in config file in section \"%s\"," \
                                             " option \"%s\":\n   %s\n" % \
                                             (section, varname, str(error)))
                            varlist[varname] = default
                    else:
                        varlist[varname] = default
            result[section] = varlist
        return result

    def __defaults(self):
        result = {}

        for section, vars in self.config_info.items():
            varlist = {}
            for varname, descr, type, default, extra_info in vars:
                varlist[varname] = default
            result[section] = varlist
        return result

    def __convert_value_to_string(self, type, value):
        if type == CONF_TYPE_BOOLEAN:
            if value:
                return "Yes"
            else:
                return "No"
        elif type == CONF_TYPE_FLOAT:
            return "%.4f" % value
        elif type in [CONF_TYPE_PATH, CONF_TYPE_MULTIPLE_FILES]:
            if len(value) == 0:
                return ""
            else:
                if sys.platform.startswith("win"):
                    return string.join(value, ";")
                else:
                    return string.join(value, ":")
        elif type == CONF_TYPE_MULTIPLE_CHOICE:
            return string.join(value, ",")
        else:
            return str(value)

    def __convert_string_to_value(self, type, valstr, extra_info = None):
        if type == CONF_TYPE_INTEGER:
            return int(valstr)
        elif type == CONF_TYPE_BOOLEAN:
            if valstr.lower() in ["1", "true", "yes"]:
                return True
            elif valstr.lower() in ["0", "false", "no"]:
                return False
            else:
                raise ValueError("invalid literal for CONF_TYPE_BOOLEAN: %s" % valstr)
        elif type == CONF_TYPE_FLOAT:
            return float(valstr)
        elif type in [CONF_TYPE_PATH, CONF_TYPE_MULTIPLE_FILES]:
            if len(valstr) == 0:
                return []
            else:
                if sys.platform.startswith("win"):
                    return string.split(valstr, ";")
                else:
                    return string.split(valstr, ":")
        elif type == CONF_TYPE_CHOICE:
            if not valstr.lower() in [x.lower() for x in zip(*extra_info)[0]]:
                raise ValueError("invalid literal for CONF_TYPE_CHOICE: %s" % valstr)
            return valstr
        elif type == CONF_TYPE_MULTIPLE_CHOICE:
            l = string.split(valstr, ",")
            s = [x.lower() for x in extra_info]
            for v in l:
                if not v.lower() in s:
                    raise ValueError("invalid literal for CONF_TYPE_MULTIPLE_CHOICE: %s" % v)
            return l
        else:
            return valstr

config = Config(configuration_info)
