# -*- coding: iso-8859-1 -*-

# Mattricks, a free helper program for hattricks players.
# Copyright (C) 2003 Mattricks Team
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import string

from Common import *
import Terms
import CommonData
import XMLParse
from Database import databases

def get_match_list(team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()

    mlist = databases.get(None, False, "Mattricks.MatchList+%d" % team_id)
    if mlist is None or mlist == "":
        mlist = []
    else:
        mlist = [int(x) for x in mlist.split(",")]
    return mlist

def set_match_list(match_list = [], team_id = None):
    if team_id is None:
        team_id = CommonData.get_current_teamid()

    mlstr = string.join([str(x) for x in match_list], ",")
    databases.sets(None, False, {("Mattricks.MatchList+%d" % team_id): mlstr})

class CurrentMatches:
    def __init__(self, team_id = None, force_update = False, progress_dialog = True):
        if team_id is None:
            team_id = CommonData.get_current_teamid()

        self.__extract_data(team_id, force_update, progress_dialog)
        self.__update_match_list()

    def __extract_data(self, team_id, f_upd, p_dlg):

        data = XMLParse.get(["match list", team_id], force_update=f_upd, progress_dialog = p_dlg)

        self.team_id = data.get("Team.TeamID", None)
        self.team_name = data.get("Team.TeamName", "")

        self.match = []
        self.match_d = {}

        if self.team_id is None:
            return

        keys = ["MatchID", "HomeTeam.HomeTeamID", "HomeTeam.HomeTeamName",
                "AwayTeam.AwayTeamID", "AwayTeam.AwayTeamName",
                "MatchDate", "MatchType", "HomeGoals", "AwayGoals",
                "Status", "OrdersGiven"]

        match_indices = CommonData.get_xml_list_indices("Team.MatchList.Match", data)
        for mindex in match_indices:
            mid = data.get("Team.MatchList.Match%s.MatchID" % mindex)
            mdata = {}
            for key in keys:
                mdata[key] = data.get("Team.MatchList.Match%s.%s" % (mindex, key), None)
            mdata = CommonData.DataStore(mdata, True)
            self.match.append(mdata)
            self.match_d[mid] = mdata

        CommonData.DataStore.set_comparison_key("MatchDate")
        self.match.sort()

    def __update_match_list(self):
        if self.team_id is None:
            return

        mlist = get_match_list(self.team_id)

        for m in self.match:
            if m.get("Status", "UPCOMING") == "FINISHED":
                mid = m.get("MatchID", None)
                if mid is not None and mid not in mlist:
                    mlist.append(mid)

        set_match_list(mlist, self.team_id)

    def GetNextMatch(self):
        for m in self.match:
            if m.get("Status", "FINISHED") in ["ONGOING", "UPCOMING"]:
                return m.get("MatchID")
        return None

class MatchDetails:

    rating_types = {"mf": "Midfield", "rd": "RightDef", "md": "MidDef",
                    "ld": "LeftDef", "ra": "RightAtt", "ma": "MidAtt",
                    "la": "LeftAtt"}

    def __init__(self, match_id):
        self.__extract_data(match_id)

    def __extract_data(self, match_id):

        data = XMLParse.get(["match details", match_id])

        self.match_id = data.get("Match.MatchID")
        self.match_date = data.get("Match.MatchDate")
        self.arena = (data.get("Match.Arena.ArenaID"),
                      data.get("Match.Arena.ArenaName"),
                      data.get("Match.Arena.WeatherID"),
                      data.get("Match.Arena.SoldTotal"),
                      data.get("Match.Arena.SoldTerraces"),
                      data.get("Match.Arena.SoldBasic"),
                      data.get("Match.Arena.SoldRoof"),
                      data.get("Match.Arena.SoldVIP"))

        self.score = (data.get("Match.HomeTeam.HomeGoals"),
                      data.get("Match.AwayTeam.AwayGoals"))

        if self.score[0] is None or self.score[1] is None:
            self.played = False
        else:
            self.played = True

        self.home_team = (data.get("Match.HomeTeam.HomeTeamID"),
                          data.get("Match.HomeTeam.HomeTeamName", "?"),
                          data.get("Match.HomeTeam.Dress"),
                          data.get("Match.HomeTeam.TeamAttitude"))
        self.away_team = (data.get("Match.AwayTeam.AwayTeamID"),
                          data.get("Match.AwayTeam.AwayTeamName", "?"),
                          data.get("Match.AwayTeam.Dress"),
                          data.get("Match.AwayTeam.TeamAttitude"))

        self.home_ratings = {}
        self.away_ratings = {}
        for rating, rating_ht in self.rating_types.items():
            self.home_ratings[rating] = data.get("Match.HomeTeam.Rating%s" % rating_ht)
            self.away_ratings[rating] = data.get("Match.AwayTeam.Rating%s" % rating_ht)

        if self.played:
            goals = data.get("Match.Scorers.xml_attrs.NumberOfGoals", 0)
            self.scorers = []
            if goals < 1:
                scorer_indices =  []
            else:
                scorer_indices = CommonData.get_xml_list_indices("Match.Scorers.Goal", data)
            for goal in scorer_indices:
                id = data.get("Match.Scorers.Goal%s.ScorerPlayerID" % goal)
                name = data.get("Match.Scorers.Goal%s.ScorerPlayerName" % goal, "?")
                team = data.get("Match.Scorers.Goal%s.ScorerTeamID" % goal)
                hgoals = data.get("Match.Scorers.Goal%s.ScorerHomeGoals" % goal)
                agoals = data.get("Match.Scorers.Goal%s.ScorerAwayGoals" % goal)
                minute = data.get("Match.Scorers.Goal%s.ScorerMinute" % goal)

                self.scorers.append((minute, hgoals, agoals, team, id, name))

            self.scorers.sort()

            books = data.get("Match.Bookings.xml_attrs.NumberOfBookings", 0)
            self.bookings = []
            if books < 1:
                booking_indices =  []
            else:
                booking_indices = CommonData.get_xml_list_indices("Match.Bookings.Booking", data)
            for booking in booking_indices:
                id = data.get("Match.Bookings.Booking%s.BookingPlayerID" % booking)
                name = data.get("Match.Bookings.Booking%s.BookingPlayerName" % booking, "?")
                team = data.get("Match.Bookings.Booking%s.BookingTeamID" % booking)
                type = data.get("Match.Bookings.Booking%s.BookingType" % booking)
                minute = data.get("Match.Bookings.Booking%s.BookingMinute" % booking)

                self.bookings.append((minute, type, team, id, name))
            self.bookings.sort()

class MatchLineup:
    def __init__(self, match_id, team_id):
        self.__extract_data(match_id, team_id)

    def __extract_data(self, match_id, team_id):

        data = XMLParse.get(["match lineup", match_id, team_id])

        # basic data

        self.match_id = data.get("MatchID")
        self.team_id = data.get("Team.TeamID")
        self.team_name = data.get("Team.TeamName", "?")
        self.match_type = data.get("MatchType")
        self.match_date = data.get("MatchDate")
        self.home_team = (data.get("HomeTeam.HomeTeamID"), data.get("HomeTeam.HomeTeamName", "?"))
        self.away_team = (data.get("AwayTeam.AwayTeamID"), data.get("AwayTeam.AwayTeamName", "?"))
        self.match_type = data.get("MatchType")
        self.arena = (data.get("Arena.ArenaID"), data.get("Arena.ArenaName"))

        lineup_indices = CommonData.get_xml_list_indices("Team.Lineup.Player", data)

        lineup_info = {}
        for i in lineup_indices:
            id = data.get("Team.Lineup.Player%s.PlayerID" % i)
            name = data.get("Team.Lineup.Player%s.PlayerName" % i, "?")
            role = data.get("Team.Lineup.Player%s.RoleID" % i)
            rating = data.get("Team.Lineup.Player%s.RatingStars" % i)
            position = data.get("Team.Lineup.Player%s.PositionCode" % i)
            behaviour = data.get("Team.Lineup.Player%s.Behaviour" % i)

            if role is not None:
                lineup_info[role] = (id, name, rating, position, behaviour)

        self.total_rating = 0.0
        for i in range(1,22):
            if not lineup_info.has_key(i):
                lineup_info[i] = (None, None, None, None, None)

            if i <= 11 and lineup_info[i][2] is not None:
                self.total_rating += lineup_info[i][2]

        self.lineup = lineup_info

    def write_lineup_text(self, outfile):
        if self.match_id is None:
            outfile.write(_("No match lineup to print") + "\n")
            return

        outfile.write("%s %d: %s - %s\n" % (_("Match"), self.match_id,
                                          self.home_team[1], self.away_team[1]))
        if self.match_type is not None:
            outfile.write(Terms.match_type[self.match_type] + "\n")
        if self.match_date is not None:
            outfile.write("%s: %s\n" % (_("Match date"), mktimestring(self.match_date)))

        outfile.write("\n%s:\n\n" % _("Lineup for %s") % self.team_name)

        parts = []
        for i in range(1,22):
            (id, name, rating, position, behaviour) = self.lineup[i]
            if id is None:
                continue

            first_part = Terms.team_role[i]
            if behaviour is not None:
                first_part += ", " + Terms.individual_order[behaviour]
            first_part += ":"

            second_part = name

            third_part = fourth_part = ""
            if rating is not None:
                third_part = "*"*(int(rating)) + ">"*(int(rating*2) % 2)
                fourth_part = "(" + str(rating) + "*)"

            parts.append((first_part, second_part, third_part, fourth_part))

        max_len = [max([len(x[i]) for x in parts]) for i in range(4)]
        for p1, p2, p3, p4 in parts:
            line = string.ljust(p1, max_len[0] + 2) + \
                   string.ljust(p2, max_len[1] + 2) + \
                   string.ljust(p3, max_len[2] + 1) + \
                   string.ljust(p4, max_len[3]) + "\n"
            outfile.write(line)

        outfile.flush()
